{
    This module provides some common routines while operating with bitmaps.
    Created 2010-07-12
    Last modified 2010-07-13
    Current version 0.3
    Copyright (C) 2010  Mariya Fomkina <mariyafomkina@users.sourceforge.net>

    This module is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This module is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this module.  If not, see <http://www.gnu.org/licenses/>.
}

unit FastBitmapRoutines;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Graphics, LCLType, IntfGraphics, fpImage, Dialogs;

type
  PRGBArray = ^TRGBArray;
  TRGBArray = array[0..32767] of TRGBTriple;
  PRGBAArray = ^TRGBAArray;
  TRGBAArray = array[0..32767] of TRGBQuad;

procedure CopyBitmapToBitmap(SrcBitmap, DestBitmap: TBitmap);

procedure CopyRectOfBitmapToBitmapWithScale(SrcBitmap, DestBitmap: TBitmap;
                                            wk, hk: real; l, t, r, b: integer);

procedure CopyBitmapToBitmapWithScale(SrcBitmap, DestBitmap: TBitmap;
                                      wk, hk: real);

procedure DrawGridOnBitmap(SrcBitmap: TBitmap; CellW, CellH, LineWidth: byte;
                           Color: TColor);

implementation

procedure CopyBitmapToBitmap(SrcBitmap, DestBitmap: TBitMap);
var
  w1, h1, w2, h2: integer;
  j, i: integer;
  SrcIntfImg, TempIntfImg: TLazIntfImage;
  ImgHandle, ImgMaskHandle: HBitmap;
  RowS24, RowD24: PRGBArray;
  RowS32, RowD32: PRGBAArray;
begin
  w1 := SrcBitmap.Width;
  h1 := SrcBitmap.Height;
  w2 := w1;
  h2 := h1;

  SrcIntfImg := TLazIntfImage.Create(0,0);
  SrcIntfImg.LoadFromBitmap(SrcBitmap.Handle,SrcBitmap.MaskHandle);
  TempIntfImg := TLazIntfImage.Create(0,0);
  TempIntfImg.DataDescription := SrcIntfImg.DataDescription;
  TempIntfImg.SetSize(w2, h2);

  if TempIntfImg.DataDescription.BitsPerPixel = 24 then
    for j := 0 to h1 - 1 do
      begin
        RowS24 := SrcIntfImg.GetDataLineStart(j);
        RowD24 := TempIntfImg.GetDataLineStart(j);
        for i := 0 to w1 - 1 do
          RowD24^[i] := RowS24^[i];
      end
  else
    for j := 0 to h1 - 1 do
      begin
        RowS32 := SrcIntfImg.GetDataLineStart(j);
        RowD32 := TempIntfImg.GetDataLineStart(j);
        for i := 0 to w1 - 1 do
          RowD32^[i] := RowS32^[i];
      end;

  TempIntfImg.CreateBitmaps(ImgHandle, ImgMaskHandle, false);
  DestBitmap.Handle := ImgHandle;
  DestBitmap.MaskHandle := ImgMaskHandle;
  SrcIntfImg.Destroy;
  TempIntfImg.Destroy;
end;

procedure CopyRectOfBitmapToBitmapWithScale(SrcBitmap, DestBitmap: TBitMap;
                                             wk, hk: real; l, t, r, b: integer);
var
  w1, h1, w2, h2: integer;
  j, i, k, s: integer;
  SrcIntfImg, TempIntfImg: TLazIntfImage;
  ImgHandle, ImgMaskHandle: HBitmap;
  RowS24, RowD24: PRGBArray;
  RowS32, RowD32: PRGBAArray;
begin
  if r > SrcBitmap.Width then
    r := SrcBitmap.Width;
  if b > SrcBitmap.Height then
    b := SrcBitmap.Height;
  w1 := r - l;
  h1 := b - t;
  w2 := Round (w1 * wk);
  h2 := Round (h1 * hk);

  SrcIntfImg := TLazIntfImage.Create(0,0);
  SrcIntfImg.LoadFromBitmap(SrcBitmap.Handle,SrcBitmap.MaskHandle);
  TempIntfImg := TLazIntfImage.Create(0,0);
  TempIntfImg.DataDescription := SrcIntfImg.DataDescription;
  TempIntfImg.SetSize(w2, h2);

  if TempIntfImg.DataDescription.BitsPerPixel = 24 then
    for j := 0 to h1 - 1 do
      begin
        RowS24 := SrcIntfImg.GetDataLineStart(j + t);
        for s := Round (hk * j) to Round (hk * (j + 1)) - 1 do
          begin
            RowD24 := TempIntfImg.GetDataLineStart(s);
            for i := 0 to w1 - 1 do
              for k := Round (wk * i) to Round (wk * (i + 1)) - 1 do
                RowD24^[k] := RowS24^[i + l];
          end;
      end
  else
    for j := 0 to h1 - 1 do
      begin
        RowS32 := SrcIntfImg.GetDataLineStart(j + t);
        for s := Round (hk * j) to Round (hk * (j + 1)) - 1 do
          begin
            RowD32 := TempIntfImg.GetDataLineStart(s);
            for i := 0 to w1 - 1 do
              for k := Round (wk * i) to Round (wk * (i + 1)) - 1 do
                RowD32^[k] := RowS32^[i + l];
          end;
      end;

  TempIntfImg.CreateBitmaps(ImgHandle, ImgMaskHandle, false);
  DestBitmap.Handle := ImgHandle;
  DestBitmap.MaskHandle := ImgMaskHandle;
  SrcIntfImg.Destroy;
  TempIntfImg.Destroy;
end;

procedure CopyBitmapToBitmapWithScale(SrcBitmap, DestBitmap: TBitMap;
                                      wk, hk: real);
begin
  CopyRectOfBitmapToBitmapWithScale(SrcBitmap, DestBitmap, wk, hk, 0, 0,
                                    SrcBitmap.Width, SrcBitmap.Height);
end;

procedure DrawGridOnBitmap(SrcBitmap: TBitmap; CellW, CellH, LineWidth: byte;
                           Color: TColor);
var
  cl: TFPColor;
  SrcIntfImg: TLazIntfImage;
  ImgHandle, ImgMaskHandle: HBitmap;
  i, j, k: integer;
begin
  SrcIntfImg := TLazIntfImage.Create(0,0);
  SrcIntfImg.LoadFromBitmap(SrcBitmap.Handle,SrcBitmap.MaskHandle);
  cl := TColorToFPColor(Color);

  for k := 0 to (LineWidth - 1) div 2 do
    for i := 0 to SrcIntfImg.Width - 1 do
      SrcIntfImg.Colors[i, k] := cl;
  for j := 0 to SrcIntfImg.Height - 1 do
    for k := 0 to (LineWidth - 1) div 2 do
      SrcIntfImg.Colors[k, j] := cl;
  if SrcIntfImg.Height mod CellH = 0 then
    for k := - (LineWidth div 2) to -1 do
      for i := 0 to SrcIntfImg.Width - 1 do
        SrcIntfImg.Colors[i, SrcIntfImg.Height + k] := cl;
  if SrcIntfImg.Width mod CellW = 0 then
    for j := 0 to SrcIntfImg.Height - 1 do
      for k := - (LineWidth div 2) to -1 do
        SrcIntfImg.Colors[SrcIntfImg.Width + k, j] := cl;

  for j := 1 to SrcIntfImg.Height - 1 do
    if j mod CellH = 0 then
      for k := - (LineWidth div 2) to (LineWidth - 1) div 2 do
        for i := 1 to SrcIntfImg.Width - 1 do
          SrcIntfImg.Colors[i, j - k] := cl
    else
      for i := 1 to (SrcIntfImg.Width - 1) div CellW do
        for k := - (LineWidth div 2) to (LineWidth - 1) div 2 do
          SrcIntfImg.Colors[i * CellW + k, j] := cl;

  SrcIntfImg.CreateBitmaps(ImgHandle, ImgMaskHandle, false);
  SrcBitmap.Handle := ImgHandle;
  SrcBitmap.MaskHandle := ImgMaskHandle;
  SrcIntfImg.Destroy;
end;

end.

