{
    Scheme Maker is an application designed to enable you
    to create knitting and cross-stitch schemes from images and photos.
    Created 2010-07-05
    Last modified 2010-08-12
    Current version 1.3
    Copyright (C) 2010  Mariya Fomkina <mariyafomkina@users.sourceforge.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
}

unit PaperPreviewUnit;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  ExtCtrls, StdCtrls, Spin, FastBitmapRoutines, Mouline, Math, LCLType,
  IntfGraphics, XMLPropStorage, FPimage;

type

  { TSaveForm }

  TSaveForm = class(TForm)
    rgOrder: TRadioGroup;
    sbScheme: TScrollBox;
    iScheme: TImage;
    pOptions: TPanel;
    lP: TLabel;
    seP: TSpinEdit;
    cgGrid: TCheckGroup;
    bExit: TButton;
    bSave: TButton;
    cbKey: TCheckBox;
    cbMouline: TComboBox;
    seK: TSpinEdit;
    lK: TLabel;
    cbColor: TCheckBox;
    cbSymbol: TCheckBox;
    XMLPropStorage1: TXMLPropStorage;
    seTh: TSpinEdit;
    lTh: TLabel;
    gbKey: TGroupBox;
    bRedraw: TButton;
    procedure PrepareSymbols(Sender: TObject);
    procedure DrawSymbolsOnScheme(Sender: TObject);
    procedure DrawKey(Sender: TObject);
    procedure sePChange(Sender: TObject);
    procedure cbKeyChange(Sender: TObject);
    procedure seKChange(Sender: TObject);
    procedure bRedrawClick(Sender: TObject);
    procedure bSaveClick(Sender: TObject);
    procedure bExitClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
  end; 

var
  SaveForm: TSaveForm;
  KeyWidth: integer;
  s, wk1, wk2, wk3: integer;
  sW, sH, sP: integer;
  bmSchemeT: TBitMap;
  bmKey: TBitMap;
  symarr: array [0..MoulineAmount] of TBitmap;
  symarr2: array [0..MoulineAmount] of TLazIntfImage;
  symarrkey: array [0..MoulineAmount] of TBitmap;

resourcestring
  cgItem2 = 'Central lines';
  rgoItem0 = 'color';
  rgoItem1 = 'number';
  rgoItem2 = 'amount';

implementation

uses
  Main;

{ TSaveForm }

//Creates several arrays with images for cells
procedure TSaveForm.PrepareSymbols(Sender: TObject);
var
  k: integer;
  ch: char;

  procedure AddSymbolToTable;
  begin
    if symarr[k] <> nil then
      symarr[k].Free;
    symarr[k] := TBitmap.Create;
    symarr[k].SetSize(sP, sP);
    with symarr[k] do
      begin
        if cbColor.Checked then
          Canvas.Brush.Color := ColorTable[k]
        else
          Canvas.Brush.Color := $ffffff;
        Canvas.FillRect(Canvas.ClipRect);
        if cbSymbol.Checked then
          begin
            Canvas.Font.Name := 'CrossStitch2';
            Canvas.Font.Height := Height - 3;
            if cbColor.Checked then
              if Red(Canvas.Brush.Color) + Green(Canvas.Brush.Color) +
                Blue(Canvas.Brush.Color) > 200 then
                Canvas.Font.Color := $000000
              else
                Canvas.Font.Color := $ffffff
            else
              Canvas.Font.Color := $000000;
            Canvas.TextOut(2, 2, ch);
          end;
      end;
    if symarrkey[k] <> nil then
      symarrkey[k].Free;
    symarrkey[k] := TBitmap.Create;
    symarrkey[k].SetSize(s + 1, s + 1);
    with symarrkey[k] do
      begin
        if cbColor.Checked then
          Canvas.Brush.Color := ColorTable[k]
        else
          Canvas.Brush.Color := $ffffff;
        Canvas.Rectangle(0, 0, s + 1, s + 1);
        if cbSymbol.Checked then
          begin
            Canvas.Font.Name := 'CrossStitch2';
            Canvas.Font.Height := Height - 3;
            if cbColor.Checked then
              if Red(Canvas.Brush.Color) + Green(Canvas.Brush.Color) +
                Blue(Canvas.Brush.Color) > 200 then
                Canvas.Font.Color := $000000
              else
                Canvas.Font.Color := $ffffff
            else
              Canvas.Font.Color := $000000;
            Canvas.TextOut(2, 2, ch);
          end;
      end;
    inc(ch);
    if ch = #128 then
      ch := #32;
    symarr2[k] := TLazIntfImage.Create(0, 0);
    symarr2[k].LoadFromBitmap(symarr[k].BitmapHandle,
      symarr[k].MaskHandle);
  end;

begin
  ch := #32;
  if MainForm.SchCond = scDetected then
    for k := 1 to MoulineAmount do
      begin
        if MainForm.OriginalColors[k] <> 0 then
          AddSymbolToTable;
      end
  else
    for k := 1 to MoulineAmount do
      if MainForm.LessenedColors[k] <> 0 then
        AddSymbolToTable;
end;

//Draws scheme according to selected type
procedure TSaveForm.DrawSymbolsOnScheme(Sender: TObject);
var
  i, j, w, h, k: integer;
  SrcIntfImg, TempIntfImg: TLazIntfImage;
  ImgHandle, ImgMaskHandle: HBitmap;
begin
  PrepareSymbols(Self);

  SrcIntfImg := TLazIntfImage.Create(0,0);
  SrcIntfImg.LoadFromBitmap(MainForm.iTemp.Picture.Bitmap.Handle,
    MainForm.iTemp.Picture.Bitmap.MaskHandle);
  TempIntfImg := TLazIntfImage.Create(0,0);
  TempIntfImg.DataDescription := SrcIntfImg.DataDescription;
  TempIntfImg.SetSize(bmSchemeT.Width,
    bmSchemeT.Height);

  for j := 0 to SrcIntfImg.Height - 1 do
    for i := 0 to SrcIntfImg.Width - 1 do
      begin
        k := GetNearestColorIndex(SrcIntfImg.Colors[i, j]);
        for h := 0 to sP - 1 do
          for w := 0 to sP - 1 do
            TempIntfImg.Colors[sP * i + w, sP * j + h] :=
              symarr2[k].Colors[w, h];
      end;

  TempIntfImg.CreateBitmaps(ImgHandle, ImgMaskHandle, false);
  bmSchemeT.Handle := ImgHandle;
  bmSchemeT.MaskHandle := ImgMaskHandle;
  SrcIntfImg.Destroy;
  TempIntfImg.Destroy;
end;

//Draws key near the scheme
procedure TSaveForm.DrawKey(Sender: TObject);
var
  i, w, h, k, wt: integer;
  crosssize: real;
  bm: TBitMap;
  cra: ColorRecordArray;

  procedure AddItemOriginal(i: integer);
  begin
    cra[i].ColorOrderNumber := i;
    cra[i].ColorFirmNumber := MoulineTable[i, k];
    cra[i].ColorAmount := MainForm.OriginalColors[i];
  end;

  procedure AddItemLessened(i: integer);
  begin
    cra[i].ColorOrderNumber := i;
    cra[i].ColorFirmNumber := MoulineTable[i, k];
    cra[i].ColorAmount := MainForm.LessenedColors[i];
  end;

  procedure Swap (var cr1, cr2: ColorRecord);
  var
    con: integer;
    cfn: string;
    ca: integer;
  begin
    con := cr1.ColorOrderNumber;
    cr1.ColorOrderNumber := cr2.ColorOrderNumber;
    cr2.ColorOrderNumber := con;
    cfn := cr1.ColorFirmNumber;
    cr1.ColorFirmNumber := cr2.ColorFirmNumber;
    cr2.ColorFirmNumber := cfn;
    ca := cr1.ColorAmount;
    cr1.ColorAmount := cr2.ColorAmount;
    cr2.ColorAmount := ca;
  end;

  procedure SortByOrderNumber;
  var
    l, j: integer;
  begin
    for l := 1 to MoulineAmount - 1 do
      for j := 1 to MoulineAmount - l do
        if cra[j].ColorOrderNumber > cra[j + 1].ColorOrderNumber then
          Swap (cra[j], cra[j + 1]);
  end;

  procedure SortByFirmNumber;
  var
    l, j: integer;
  begin
    for l := 1 to MoulineAmount - 1 do
      for j := 1 to MoulineAmount - l do
        if cra[j].ColorFirmNumber > cra[j + 1].ColorFirmNumber then
          Swap (cra[j], cra[j + 1]);
  end;

  procedure SortByAmount;
  var
    l, j: integer;
  begin
    for l := 1 to MoulineAmount - 1 do
      for j := 1 to MoulineAmount - l do
        if cra[j].ColorAmount < cra[j + 1].ColorAmount then
          Swap (cra[j], cra[j + 1]);
  end;

  procedure DrawItem(i: integer);
  begin
    with bmKey.Canvas do
      begin
        Draw(w, h, symarrkey[cra[i].ColorOrderNumber]);
        wt := (wk1 - TextWidth(cra[i].ColorFirmNumber)) div 2;
        TextOut(w + s + 5 + wt, h, cra[i].ColorFirmNumber);
        wt := (wk2 - TextWidth(IntToStr(cra[i].ColorAmount))) div 2;
        TextOut(w + s + wk1 + 10 + wt, h, IntToStr(cra[i].ColorAmount));
        wt := (wk3 - TextWidth(FloatToStrF(cra[i].ColorAmount *
          crosssize, ffGeneral, 3, 5))) div 2;
        TextOut(w + s + wk1 + wk2 + 15 + wt, h, FloatToStrF
          (cra[i].ColorAmount * crosssize, ffGeneral, 3, 5));
        h := h + s + (s div 4);
        if h + s + (s div 4) > iScheme.Height - 20 then
          begin
            h := 20;
            w := w + s + wk1 + wk2 + wk3 + 25;
          end;
      end;
  end;

begin
  bmKey := TBitMap.Create;
  with bmKey.Canvas do
    begin
      Font.Height := s;
      Font.Style := [fsBold];
      wk1 := TextWidth('00000000');
      wk2 := TextWidth('0000000');
      wk3 := TextWidth('0000000');
    end;
  KeyWidth := s + wk1 + wk2 + wk3 + 25;
  if MainForm.SchCond = scDetected then
    KeyWidth := KeyWidth * ((MainForm.OrigClNumber + 3) div
      (bmSchemeT.Height div (s + (s div 4))) + 1)
  else if MainForm.SchCond = scLessened then
    KeyWidth := KeyWidth * ((MainForm.LessClNumber + 3) div
      (bmSchemeT.Height div (s + (s div 4))) + 1) ;
  bmKey.Width := KeyWidth;
  bmKey.Height := iScheme.Height;

  w := 0;
  h := 20;
  k := cbMouline.ItemIndex + 1;
  crosssize := 0.01 / MainForm.fseCm.Value * (2 + 2 * 1.4142) * seTh.Value;

  bm := TBitmap.Create;
  bm.Width := 1;
  bm.Height := 1;
  bm.Canvas.Pixels[0, 0] := clWhite;
  CopyBitmapToBitmapWithScale(bm, bmKey,
    bmKey.Width, bmKey.Height);

  with bmKey.Canvas do
    begin
      Brush.Color := $ffffff;
      Font.Name := 'Arial';
      TextOut(w, h, 'Size: ' + IntToStr(MainForm.sW) + 'x' +
        IntToStr(MainForm.sH));
      h := h + s + (s div 4);
      if MainForm.SchCond = scDetected then
        TextOut(w, h, 'Colors: ' + IntToStr(MainForm.OrigClNumber))
      else
        TextOut(w, h, 'Colors: ' + IntToStr(MainForm.LessClNumber));
      h := h + s + (s div 4);
      wt := (wk1 - TextWidth(MoulineTable[0, k])) div 2;
      TextOut(w + s + 5 + wt, h, MoulineTable[0, k]);
      wt := (wk2 - TextWidth('x')) div 2;
      TextOut(w + s + wk1 + 10 + wt, h, 'x');
      wt := (wk2 - TextWidth('m')) div 2;
      TextOut(w + s + wk1 + wk2 + 15 + wt, h, 'm');
      h := h + s + (s div 4);
    end;

  if MainForm.SchCond = scDetected then
    begin
      for i := 1 to MoulineAmount do
        if MainForm.OriginalColors[i] <> 0 then
          AddItemOriginal(i)
        else
            cra[i].ColorAmount := 0;
    end
  else if MainForm.SchCond = scLessened then
    begin
      for i := 1 to MoulineAmount do
        if MainForm.LessenedColors[i] <> 0 then
          AddItemLessened(i)
        else
            cra[i].ColorAmount := 0;
    end;

  case rgOrder.ItemIndex of
    0: SortByOrderNumber;
    1: SortByFirmNumber;
    2: SortByAmount;
  end;

  for i := 1 to MoulineAmount do
    if cra[i].ColorAmount <> 0 then
      DrawItem(i);

  iScheme.Width := iScheme.Width + KeyWidth;
  iScheme.Picture.Bitmap.Width := iScheme.Width;
  iScheme.Picture.Bitmap.Canvas.Draw(iScheme.Width - KeyWidth, 0, bmKey);
end;

//Next procedures are reactions on changing some value

procedure TSaveForm.sePChange(Sender: TObject);
begin
  sP := seP.Value;
end;

procedure TSaveForm.cbKeyChange(Sender: TObject);
begin
  gbKey.Enabled := cbKey.Checked;
end;

procedure TSaveForm.seKChange(Sender: TObject);
begin
  s := seK.Value;
end;

//Next procedures are reations on button clicks

procedure TSaveForm.bRedrawClick(Sender: TObject);
var
  i: integer;
  bm: TBitmap;
begin
  Screen.Cursor := crHourGlass;

  sW := MainForm.sW;
  sH := MainForm.sH;
  bmSchemeT := TBitmap.Create;
  bmSchemeT.Width := sW * sP;
  bmSchemeT.Height := sH * sP;
  DrawSymbolsOnScheme(Self);
  if cgGrid.Checked[0] then
    DrawGridOnBitmap(bmSchemeT, sP, sP, 1, RGBToColor(0, 0, 1));
  if cgGrid.Checked[1] then
    DrawGridOnBitmap(bmSchemeT, sP * 10, sP * 10, 2, RGBToColor(0, 0, 1));
  if cgGrid.Checked[2] then
    begin
      bmSchemeT.Canvas.Pen.Color := clRed;
      bmSchemeT.Canvas.Pen.Width := 3;
      bmSchemeT.Canvas.Line(0, (bmSchemeT.Height div (2 * sP)) * sP,
        bmSchemeT.Width, (bmSchemeT.Height div (2 * sP)) * sP);
      bmSchemeT.Canvas.Line((bmSchemeT.Width div (2 * sP)) * sP, 0,
        (bmSchemeT.Width div (2 * sP)) * sP, bmSchemeT.Height);
    end;

  iScheme.Picture.Bitmap.Clear;
  iScheme.Width := bmSchemeT.Width + 40;
  iScheme.Height := bmSchemeT.Height + 40;
  bm := TBitmap.Create;
  bm.Width := 1;
  bm.Height := 1;
  bm.Canvas.Pixels[0, 0] := clWhite;
  CopyBitmapToBitmapWithScale(bm, iScheme.Picture.Bitmap,
    iScheme.Width, iScheme.Height);
  with iScheme.Picture.Bitmap.Canvas do
    begin
      Pen.Color := clBlack;
      Pen.Width := 1;
      Line(20, 19 + bmSchemeT.Height, 20 + bmSchemeT.Width,
        19 + bmSchemeT.Height);
      Line(19 + bmSchemeT.Width, 20, 19 + bmSchemeT.Width,
        20 + bmSchemeT.Height);
      Font.Height := 12;
      for i := 1 to MainForm.iTemp.Picture.Bitmap.Width div 10 do
        TextOut(20 + i * 10 * sP - TextWidth('0'), 7, IntToStr(i * 10));
      for i := 1 to MainForm.iTemp.Picture.Bitmap.Height div 10 do
        TextOut(20 - TextWidth('00'), 20 + i * 10 * sP - 6, IntToStr(i * 10));
      Font.Height := s;
      Draw(20, 20, bmSchemeT);
    end;
  if cbKey.Checked then
    DrawKey(Self);

  Screen.Cursor := crDefault;
end;

procedure TSaveForm.bSaveClick(Sender: TObject);
begin
  if MainForm.SavePictureDialog1.Execute then
    begin
      if ExtractFileExt(MainForm.SavePictureDialog1.FileName) = '' then
        if MainForm.SavePictureDialog1.FilterIndex = 1 then
          MainForm.SavePictureDialog1.FileName :=
            MainForm.SavePictureDialog1.FileName + '.png'
        else
          MainForm.SavePictureDialog1.FileName :=
            MainForm.SavePictureDialog1.FileName + '.' +
            MainForm.SavePictureDialog1.GetFilterExt;
      iScheme.Picture.SaveToFile(MainForm.SavePictureDialog1.FileName);
      MainForm.IsSaved := true;
    end;
end;

procedure TSaveForm.bExitClick(Sender: TObject);
begin
  Close;
end;

procedure TSaveForm.FormShow(Sender: TObject);
begin
  cgGrid.Items.Clear;
  cgGrid.Items.Add('1x1');
  cgGrid.Items.Add('10x10');
  cgGrid.Items.Add(cgItem2);
  cgGrid.Checked[0] := True;
  cgGrid.Checked[1] := True;
  rgOrder.Items.Clear;
  rgOrder.Items.Add(rgoItem0);
  rgOrder.Items.Add(rgoItem1);
  rgOrder.Items.Add(rgoItem2);
  sePChange(Self);
  seKChange(Self);
  bRedrawClick(Self);
  MainForm.SavePictureDialog1.Title := bSave.Caption;
end;

initialization
  {$I paperpreviewunit.lrs}

end.

